@@ -13,17 +13,17 @@
 %                  Read/Write the Magick Persistent Registry.                 %
 %                                                                             %
 %                              Software Design                                %
-%                                John Cristy                                  %
+%                                   Cristy                                    %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,22 +39,24 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/registry.h"
-#include "magick/static.h"
-#include "magick/string_.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/registry.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
 
 /*
   Forward declarations.
 */
 static MagickBooleanType
-  WriteMPRImage(const ImageInfo *,Image *);
+  WriteMPRImage(const ImageInfo *,Image *,ExceptionInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -78,37 +80,28 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
-%
 */
 static Image *ReadMPRImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-  char
-    *p;
-
-  long
-    id;
-
-  RegistryType
-    type;
-
-  size_t
-    length;
+  Image
+    *image;
 
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  if (LocaleCompare(image_info->magick,"MPRI") != 0)
-    return(GetImageFromMagickRegistry(image_info->filename,&id,exception));
-  id=strtol(image_info->filename,&p,0);
-  return((Image *) GetMagickRegistry(id,&type,&length,exception));
+  assert(exception->signature == MagickCoreSignature);
+  image=(Image *) GetImageRegistry(ImageRegistryType,image_info->filename,
+    exception);
+  if (image != (Image *) NULL)
+    (void) SyncImageSettings(image_info,image,exception);
+  return(image);
 }
 
 /*
@@ -131,31 +124,29 @@ static Image *ReadMPRImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterMPRImage method is:
 %
-%      RegisterMPRImage(void)
+%      size_t RegisterMPRImage(void)
 %
 */
-ModuleExport void RegisterMPRImage(void)
+ModuleExport size_t RegisterMPRImage(void)
 {
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("MPR");
-  entry->decoder=(DecoderHandler *) ReadMPRImage;
-  entry->encoder=(EncoderHandler *) WriteMPRImage;
-  entry->adjoin=MagickFalse;
-  entry->stealth=MagickTrue;
-  entry->description=AcquireString("Magick Persistent Registry");
-  entry->module=AcquireString("MPR");
+  entry=AcquireMagickInfo("MPR","MPR","Magick Persistent Registry");
+  entry->decoder=(DecodeImageHandler *) ReadMPRImage;
+  entry->encoder=(EncodeImageHandler *) WriteMPRImage;
+  entry->flags^=CoderAdjoinFlag;
+  entry->format_type=ImplicitFormatType;
+  entry->flags|=CoderStealthFlag;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("MPRI");
-  entry->decoder=(DecoderHandler *) ReadMPRImage;
-  entry->encoder=(EncoderHandler *) WriteMPRImage;
-  entry->stealth=MagickTrue;
-  entry->adjoin=MagickFalse;
-  entry->stealth=MagickTrue;
-  entry->description=AcquireString("Magick Persistent Registry");
-  entry->module=AcquireString("MPRI");
+  entry=AcquireMagickInfo("MPR","MPRI","Magick Persistent Registry");
+  entry->decoder=(DecodeImageHandler *) ReadMPRImage;
+  entry->encoder=(EncodeImageHandler *) WriteMPRImage;
+  entry->flags^=CoderAdjoinFlag;
+  entry->format_type=ImplicitFormatType;
+  entry->flags|=CoderStealthFlag;
   (void) RegisterMagickInfo(entry);
+  return(MagickImageCoderSignature);
 }
 
 /*
@@ -179,8 +170,8 @@ ModuleExport void RegisterMPRImage(void)
 */
 ModuleExport void UnregisterMPRImage(void)
 {
-  (void) UnregisterMagickInfo("MPR");
   (void) UnregisterMagickInfo("MPRI");
+  (void) UnregisterMagickInfo("MPR");
 }
 
 /*
@@ -200,41 +191,31 @@ ModuleExport void UnregisterMPRImage(void)
 %
 %  The format of the WriteMPRImage method is:
 %
-%      MagickBooleanType WriteMPRImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteMPRImage(const ImageInfo *image_info,
+%        Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
-%    o image_info: The image info.
+%    o image_info: the image info.
 %
 %    o image:  The image.
 %
+%    o exception: return any errors or warnings in this structure.
 %
 */
-static MagickBooleanType WriteMPRImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteMPRImage(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
-  ExceptionInfo
-    exception;
-
-  Image
-    *registry_image;
-
-  long
-    id;
+  MagickBooleanType
+    status;
 
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
+  magick_unreferenced(image_info);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  GetExceptionInfo(&exception);
-  registry_image=GetImageFromMagickRegistry(image->filename,&id,&exception);
-  DestroyExceptionInfo(&exception);
-  if (registry_image != (Image *) NULL)
-    {
-      (void) DeleteMagickRegistry(id);
-      registry_image=DestroyImage(registry_image);
-    }
-  id=SetMagickRegistry(ImageRegistryType,image,sizeof(Image),&image->exception);
-  return((MagickBooleanType) (id >= 0));
+  status=SetImageRegistry(ImageRegistryType,image->filename,image,exception);
+  return(status);
 }
